==> snapstraction-0.3.5a1-py2-none-any.whl: snapstraction/mesh.py <==
"""SNAPstraction mesh/network ops"""

import binascii
import logging

import random

from snaplib import Snap2Decoder

from n2nupgrade import n2n
from n2nupgrade.exceptions import RpcTimeoutException

from tornado.concurrent import Future
from tornado.gen import coroutine, sleep, Return
import tornado.ioloop

from .node_cache import NodeCache
from .sniffer.sniffer import Sniffer

log = logging.getLogger(__name__)


def sniff(output_func, profile, network, channel=None):
    """Start the hardware sniffer"""

    if channel is None:
        channel = network.channel

    log.info("Start sniffing on device %s and channel %s",
             profile.device, channel)

    def callback(event=None):
        sniffer_callback_no_fmt(output_func, event)

    sniffer = Sniffer()
    sniffer.on_event(callback)
    sc_profile = profile.as_sc_params()
    sc_net = network.as_sc_params()
    sniffer.set_encryption(sc_net.encryption_type, sc_net.encryption_key)
    sniffer.open(sc_profile.port)
    sniffer.start(channel)
    tornado.ioloop.IOLoop.current().start()


def sniffer_callback_no_fmt(output_func, event=None):
    """Sniffer on packet callback"""
    try:
        (headers, sections) = Snap2Decoder.get_decoded_pkt_sections(event.data, decrypted=True)
        pkt = dict(zip(headers, sections))
        output_func(pkt)
    except AssertionError:
        pass


@coroutine
def unicast_ping(gus, data, timeout, output_func, progress_cb=None, finished_cb=None):
    """Ping the mesh w/ Unicast vmstats"""
    with NodeCache() as cache:

        class nonlocal:
            pass

        nonlocal.found_count = 0
        nonlocal.last_addr = ''

        def on_vmStat(src, *args):
            """tellVmStat callback"""
            (word, d) = args

            word = word & 0xFFFF

            code = word & 0xFF
            hibyte = word >> 8

            # Make sure it's our tellVmStat...
            if code == 5:
                lq = hibyte
                data[src] = lq
                cache.add(src)
                nonlocal.found_count += 1
                nonlocal.last_addr = src

        gus.on_rpc('tellVmStat', on_vmStat)
        gus.mcast_rpc('vmStat', 5, timeout, group=1, ttl=timeout if timeout < 255 else 50)

        secs = 0
        while secs < timeout:
            yield sleep(1)
            if progress_cb:
                progress_cb(secs, nonlocal.found_count, nonlocal.last_addr)
            secs += 1

        if finished_cb:
            finished_cb()


@coroutine
def dmcast_ping(gus, data, timeout, output_func, response_name=None, progress_cb=None, finished_cb=None):
    """Ping the mesh with directed multicast"""
    with NodeCache() as cache:
        if not response_name:
            response_name = "dmPing%06d" % random.randint(0, 999999)

        log.debug("Looking for response: %s", response_name)

        class nonlocal:
            pass

        nonlocal.found_count = 0
        nonlocal.last_addr = ''

        def on_response(src, *args):
            if src not in data:
                data[src] = args
                cache.add(src)
                nonlocal.found_count += 1
                nonlocal.last_addr = src

        gus.on_rpc(response_name, on_response)

        secs = 0
        while secs < timeout:
            yield gus.mcast_rpc('dmcastRpc', binascii.unhexlify(gus.address), 1, timeout, 50, response_name, 'ping', group=1, ttl=timeout if timeout < 255 else 20)
            yield sleep(1)
            if progress_cb:
                progress_cb(secs, nonlocal.found_count, nonlocal.last_addr)
            secs += 1

        if finished_cb:
            finished_cb()


@coroutine
def listen(gus, output_func, _timeout=None):
    """Listen for packets that come our way"""
    def any_rpc(src, func, *args):
        output_func({'src': src, 'func': func, 'args': args})

    def any_data(src, data):
        output_func({'src': src, 'data': data})

    gus.on_any_rpc(any_rpc)
    gus.on_data(any_data)

    log.debug("Registered, waiting")

    if _timeout:
        yield sleep(_timeout)
    else:
        yield Future()


@coroutine
def listen_for_intercepted_output(gus, output_func, _timeout=None):
    """Listen for intercepted output from nodes"""
    def on_data(src, data):
        output_func({'node': src, 'packet': data})

    gus.on_data(on_data)

    if _timeout:
        yield sleep(_timeout)
    else:
        yield Future()


@coroutine
def take_node_census_recursive(gus, root_node):
    """Take the census of the mesh"""
    # The n2nupgrader expects the SCF instance to have a custom bridge_addr property,
    # so we will need to set it manually until SCF supports obtaining the bridge address
    gus._scf.bridge_addr = gus.bridge_address

    # The following algorithm is used to poll the mesh topology:
    # As nodes are polled for their topology, add them to the queried_nodes set.
    # As nodes in the topology are discovered, add them to the remaining_nodes
    # set, unless they are already part of the queried_nodes set, in which case
    # ignore them. This ensures that each node in the mesh is polled exactly once.
    topo_taker = n2n.TopologyTaker(gus._scf)
    remaining_nodes = set([binascii.unhexlify(root_node)])
    queried_nodes = set()
    while len(remaining_nodes) > 0:
        # Take the topoogy of the next node in the queue and mark it as
        # already having had its topology taken
        next_node = remaining_nodes.pop()
        try:
            yield topo_taker.take_node_census(next_node)
        except RpcTimeoutException:
            # Ignore timeout exceptions
            pass
        queried_nodes.add(next_node)

        # Add to the queue any fringe nodes in the topology that nave not had
        # their topology taken yet
        topo_nodes = [edge.master_addr for edge in topo_taker.topo_edges] + [edge.neighbor_addr for edge in topo_taker.topo_edges]
        for node_addr in topo_nodes:
            if node_addr not in queried_nodes:
                remaining_nodes.add(node_addr)

    # Process the topology edge list into a topology graph
    topo_processor = n2n.TopologyProcessor()
    topology = topo_processor.generate_topology(topo_taker.topo_edges)

    def format_addr(node_addr):
        """Make address human-readable"""
        return binascii.hexlify(node_addr)

    formatted_topology = {format_addr(node_addr): {format_addr(neighbor_addr): link_quality for neighbor_addr, link_quality in topology_data['neighbors'].iteritems()} for node_addr, topology_data in topology.iteritems()}
    raise Return(formatted_topology)


@coroutine
def poll_topology(gus, data):
    """Gather the topology"""
    log.info("Gathering topology")

    data['topology'] = yield take_node_census_recursive(gus, gus.bridge_address)

==> snapstraction-0.8.1-py2-none-any.whl: snapstraction/mesh.py <==
"""SNAPstraction mesh/network ops"""

import binascii
import logging

import random

from tornado.concurrent import Future
from tornado.gen import coroutine, sleep, Return

from .node_cache import NodeCache

log = logging.getLogger(__name__)


@coroutine
def unicast_ping(gus, count=3, ttl=5, progress_cb=None, finished_cb=None):
    """Ping the mesh w/ Unicast vmstats"""

    data = {}

    with NodeCache() as cache:

        class nonlocal:
            pass

        nonlocal.found_count = 0
        nonlocal.last_addr = ''

        def on_vmStat(src, *args):
            """tellVmStat callback"""
            (word, d) = args

            word = word & 0xFFFF

            code = word & 0xFF
            hibyte = word >> 8

            # Make sure it's our tellVmStat...
            if code == 5:
                lq = hibyte
                data[src] = lq
                cache.add(src)
                nonlocal.found_count += 1
                nonlocal.last_addr = src

        gus.on_rpc('tellVmStat', on_vmStat)

        secs = 0
        while secs < count:
            gus.mcast_rpc('vmStat', 5, count, group=1, ttl=ttl)
            yield sleep(1)
            if progress_cb:
                progress_cb(secs, nonlocal.found_count, nonlocal.last_addr)
            secs += 1

        if finished_cb:
            finished_cb()

    raise Return(data)


@coroutine
def dmcast_ping(gus, count=3, ttl=5, response_name=None, progress_cb=None, finished_cb=None):
    """Ping the mesh with directed multicast"""
    data = {}
    with NodeCache() as cache:
        if not response_name:
            response_name = "dmPing%06d" % random.randint(0, 999999)

        log.debug("Looking for response: %s", response_name)

        class nonlocal:
            pass

        nonlocal.found_count = 0
        nonlocal.last_addr = ''

        def on_response(src, *args):
            if src not in data:
                data[src] = args[0]
                cache.add(src)
                nonlocal.found_count += 1
                nonlocal.last_addr = src

        gus.on_rpc(response_name, on_response)

        secs = 0
        while secs < count:
            yield gus.mcast_rpc('dmCallout', binascii.unhexlify(gus.address), 1, ttl, 50, response_name, 'getLq', group=1, ttl=ttl)
            yield sleep(1)
            if progress_cb:
                progress_cb(secs, nonlocal.found_count, nonlocal.last_addr)
            secs += 1

        if finished_cb:
            finished_cb()

    raise Return(data)


@coroutine
def listen(gus, output_func, _timeout=None):
    """Listen for packets that come our way"""
    def any_rpc(src, func, *args):
        output_func({'src': src, 'func': func, 'args': args})

    def any_data(src, data):
        output_func({'src': src, 'data': data})

    gus.on_any_rpc(any_rpc)
    gus.on_data(any_data)

    log.debug("Registered, waiting")

    if _timeout:
        yield sleep(_timeout)
    else:
        yield Future()


@coroutine
def listen_for_intercepted_output(gus, output_func, _timeout=None):
    """Listen for intercepted output from nodes"""
    def on_data(src, data):
        output_func({'node': src, 'packet': data})

    gus.on_data(on_data)

    if _timeout:
        yield sleep(_timeout)
    else:
        yield Future()
